@@ -124,13 +124,13 @@ pandoc_arg = function(x) {
   nms = names(x)
   if (any(grepl('\n', x))) {
     # one argument used multiple times, e.g. --bibliography
-    x = str_split(x, '\n')
+    x = stringr::str_split(x, '\n')
     nms = rep(nms, sapply(x, length))
     x = unlist(x)
   }
   a1 = nchar(nms) == 1L
-  paste(ifelse(a1, '-', '--'), nms,
-        ifelse(x == '', '', ifelse(a1, ' ', '=')), x, sep = '', collapse = ' ')
+  paste0(ifelse(a1, '-', '--'), nms,
+         ifelse(x == '', '', ifelse(a1, ' ', '=')), x, collapse = ' ')
 }
 # identify pandoc config in markdown comments
 pandoc_cfg = function(x) {
