@@ -13,6 +13,8 @@ auto_exts = c(
 
   Cairo_pdf = 'pdf', Cairo_png = 'png', Cairo_ps = 'eps', Cairo_svg = 'svg',
 
+  svglite = 'svg',
+
   tikz = 'tikz'
 )
 
@@ -105,6 +107,8 @@ save_plot = function(plot, name, dev, width, height, ext, dpi, options) {
     Cairo_ps = load_device('Cairo_ps', 'cairoDevice'),
     Cairo_svg = load_device('Cairo_svg', 'cairoDevice'),
 
+    svglite = load_device('svglite', 'svglite'),
+
     tikz = function(...) {
       tikz_dev(..., sanitize = options$sanitize, standAlone = options$external)
     },
@@ -124,7 +128,7 @@ plot2dev = function(plot, name, dev, device, path, width, height, options) {
 
   # compile tikz to pdf
   if (dev == 'tikz' && options$external) {
-    unlink(pdf.plot <- paste(name, '.pdf', sep = ''))
+    unlink(pdf.plot <- paste0(name, '.pdf'))
     owd = setwd(dirname(path))
     # add old wd to TEXINPUTS (see #188)
     oti = Sys.getenv('TEXINPUTS'); on.exit(Sys.setenv(TEXINPUTS = oti))
@@ -307,8 +311,8 @@ plot_crop = function(x, quiet = !opts_knit$get('progress')) {
   }
   # see this post for why use shell() on Windoz:
   # http://comments.gmane.org/gmane.comp.lang.r.devel/38113
-  if (is_windows() && cmd == 'convert') {
-    shell(paste(cmd, args))  # no way to quiet cmd output on Windoz
+  if (is_windows()) {
+    shell(paste(c(cmd, args), collapse = ' '))  # no way to quiet cmd output on Windoz
   } else {
     system2(cmd, args = args, stdout = if (quiet) FALSE else "")
   }
@@ -319,3 +323,36 @@ plot_crop = function(x, quiet = !opts_knit$get('progress')) {
 showtext = function(show) {
   if (isTRUE(show)) getFromNamespace('showtext.begin', 'showtext')()
 }
+
+#' Embed external images in \pkg{knitr} documents
+#'
+#' When plots are not generated from R code, there is no way for \pkg{knitr} to
+#' capture plots automatically. In this case, you may generate the images
+#' manually and pass their file paths to this function to include them in the
+#' output. The major advantage of using this function is that it is portable in
+#' the sense that it works for all document formats that \pkg{knitr} supports,
+#' so you do not need to think if you have to use, for example, LaTeX or
+#' Markdown syntax, to embed an external image. Chunk options related to
+#' graphics output that work for normal R plots also work for these images, such
+#' as \code{out.width} and \code{out.height}.
+#' @param path a character vector of image paths
+#' @param auto_pdf whether to use PDF images automatically when the output
+#'   format is LaTeX, e.g. \file{foo/bar.png} will be replaced by
+#'   \file{foo/bar.pdf} if the latter exists; this can be useful since normally
+#'   PDF images are of higher qualities than raster images like PNG when the
+#'   output is LaTeX/PDF
+#' @note This function is supposed to be used in R code chunks or inline R code
+#'   expressions. You are recommended to use forward slashes (\verb{/}) as path
+#'   separators instead of backslashes in the image paths.
+#' @return The same as the input character vector \code{path} but it is marked
+#'   with special internal S3 classes so that \pkg{knitr} will convert the file
+#'   paths to proper output code according to the output format.
+#' @export
+include_graphics = function(path, auto_pdf = TRUE) {
+  if (auto_pdf && is_latex_output()) {
+    path2 = sub_ext(path, 'pdf')
+    i = file.exists(path2)
+    path[i] = path2[i]
+  }
+  structure(path, class = c('knit_image_paths', 'knit_asis'))
+}
